% ExperimentScript3.m
%
% NOTE1: Before executing this script, make sure that the required wired and
% wireless connections have been established succesfully in part
% ExperimentScript2.m. This script MUST be executed after
% ExperimentScript2.m since it uses the same parameters throughout the
% lines and same connections to communicate with the microcontroller and 
% the Actor PC. 
% NOTE2: Similar to the ExperimentScrip2.m this script also expects an
% audio file to be loaded. If it is not provided, errors will occur.
%%%%%%%%%%%%%%%%%%%%%%%%%% Script Description %%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ExperimentScript3.m creates an active window on the participant display
% (a second display that is connected to the experimenter PC) and executes
% the required functions and commands to:
% 1. Change the state of the LEDs via Arduino connection
% 2. Display trials to the participants,
% 3. Record their response trajectories,
% 4. Help the experimenter to control the breaks between blocks (i.e., the
% script requires a key input from the experiment to switch between blocks)
% 5. Provide information about the current block,trial to the experimenter
% to double-check everything is OK.

% File name definitions
% The data is saved in .xlsx and .mat formats
experiment3_xlsx_file  =  convertStringsToChars((convertCharsToStrings(id_str) + "_ExperimentScript3_Data.xlsx"));
experiment3_mat_file   = convertStringsToChars(( convertCharsToStrings(id_str) + "_ExperimentScript3_Data" +".mat"));

% The data files will be saved in the same folder that was created in
% ExperimentScript1.m Lines XX.
button.dimensionLow = dimensionLow;
button.dimensionHigh = dimensionHigh;

% Randomization of Blocks & Stimuli Order
[expBlockOrderNo, experimentBlockOrder] = RandomizeBlocks(pID);
[trialOrderNo, actionOrder]= RandomizeTrials(pID);
trialCounter = 1;
% Psychtoolbox start-up
Screen('Preference', 'SkipSyncTests',1); % Included to skip Synchronization tests
clear wPtr;
[wPtr, rect] = Screen('OpenWindow', screenNum);
feature('DefaultCharacterSet', 'UTF8');
Screen('Preference', 'TextEncodingLocale', 'UTF-8');
FlipInterval = Screen('GetFlipInterval',wPtr);
slack = FlipInterval/2;
Screen('FillRect', wPtr, background);
[VBLt,~] =Screen('Flip',wPtr);
Screen('ConstrainCursor', wPtr, 1, rect);
HideCursor;
textureCaller = 1;
textures = GenerateTextures(wPtr,textureCaller);

% Experiment Part 3 begins as the lines below here start to be executed
Screen('FillRect',wPtr,white);
Screen('DrawTexture',wPtr,textures.fixation,[],pageRect);
Screen('Flip',wPtr);
disp(("-------------------------------------------------------------------------------------------"+newline));
disp("Input required from the executor:"+newline+...
    "Press ESC key when the participant adjust their position."+newline);
while 1
    [keyIsDown, secs, keyCode] = KbCheck;
    if keyIsDown
        if keyCode(escKey)
            % If ESC key is pressed, exit the loop and continue to
            % execution of the script.
            break ;
        end
    end
end
disp(("-------------------------------------------------------------------------------------------"+newline));
disp("Input required from the executor:"+newline+...
    ">> Press ESC key when the stage ready information is received."+newline);


while 1
    % Play an audio until the stage ready gesture is displayed on the
    % Camera PC and the executor presses ESC key
    [keyIsDown, secs, keyCode] = KbCheck;
    if keyIsDown
        if keyCode(escKey)
            break ;
        end
    end
    if(isplaying(audio_player) == 0)
        play(audio_player);
    end
end
stop(audio_player);
rightChoiceID = 3; rightChoice ="Low";
leftChoiceID = 4; leftChoice = "High";
%% Experiment3 Begins Here
num_blocks = experimentBlockOrder;
curr_block = 0; % Experiment
trial_order_str = actionOrder;

% Instruction Pages 
Screen('DrawTexture',wPtr,textures.page31_new,[],pageRect);
Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
Screen('Flip', wPtr, 0);
RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);

Screen('DrawTexture',wPtr,textures.page32,[],pageRect);
Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
Screen('Flip', wPtr, 0);
RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);

Screen('DrawTexture',wPtr,textures.page33,[],pageRect);
Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
Screen('Flip', wPtr, 0);
RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);

Screen('DrawTexture',wPtr,textures.page34,[],pageRect);
Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
Screen('Flip', wPtr, 0);
RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);

Screen('DrawTexture',wPtr,textures.page35,[],pageRect);
Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
Screen('Flip', wPtr, 0);
RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);

Screen('DrawTexture',wPtr,textures.page36,[],pageRect);
Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
Screen('Flip', wPtr, 0);
RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);
HideCursor;

for blocks=1:length(num_blocks)
    % The control block below is included to control the messages sent to
    % the Actor PC prior to the beginning of a new block. The same actor
    % cannot perform subsequent two blocks.
    if blocks == 1
        if experimentBlockOrder(blocks) == 1 ||  experimentBlockOrder(blocks) == 3
            % First Block Belongs to Actor2
            actor = "Actor2";
            actorID = 1;
            write(machineServer,"It's the turn of ACTOR2.\n");
        else
            % First Block Belongs to Actor1
            actor = "Actor1";
            actorID = 2;
            write(machineServer,"It's the turn of ACTOR1.\n");

        end
    else
        % Continuing blocks, the current actor needs to change.
        previousBlock = experimentBlockOrder(blocks-1); % previous block id
        currBlock = experimentBlockOrder(blocks); % current block id
        if previousBlock == 2 || previousBlock == 4
            % previous block belongs to Actor1
            % currect block belongs to Actor2
            actor = "Actor2";
            actorID = 1;
            write(machineServer,"It's the turn of ACTOR2.\n");
        else
            % previous block belogs to Actor2
            % current block belongs to Actor1
            actor = "Actor1";
            actorID = 2;
            write(machineServer,"It's the turn of ACTOR1.\n");
        end
    end
    % The following switch-case block is included to change the
    % experiment parameters used by various functions to accompany
    % changes in actor condition.
    switch experimentBlockOrder(blocks)
        case 1
            % Exp 3 Agency Actor 2
            blockDim_str = "Agency";
            dim = 1;
            actor = "Actor2";
            actorID = 1;
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp(("Note to the executor:"+newline+...
                " The upcoming block is: " + blockDim_str + " " + actor +newline));
            Screen('FillRect',wPtr,background);
            Screen('DrawTexture',wPtr,textures.page37,[],pageRect);
            Screen('Flip', wPtr, 0);
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp("Input required from the executor:"+newline+...
                ">> Press ESC key to continue.");
            while 1
                [keyIsDown, secs, keyCode] = KbCheck;
                if keyIsDown
                    if keyCode(escKey)
                        break ;
                    end
                end
                if(isplaying(audio_player) == 0)
                    play(audio_player);
                end
            end
            stop(audio_player);
            Screen('DrawTexture',wPtr,textures.page38,[],pageRect);
            Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
            Screen('Flip', wPtr, 0);
            RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);
        case 2
            % Exp3 Task - Agency Actor1
            blockDim_str = "Agency";
            dim = 1;
            actor = "Actor1";
            actorID = 0;
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp(("Note to the executor:"+newline+...
                " The upcoming block is: " + blockDim_str + " " + actor +newline));
            Screen('FillRect',wPtr,background);
            Screen('DrawTexture',wPtr,textures.page37,[],pageRect);
            Screen('Flip', wPtr, 0);
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp("Input required from the executor:"+newline+...
                ">> Press ESC key to continue.");
            while 1
                [keyIsDown, secs, keyCode] = KbCheck;
                if keyIsDown
                    if keyCode(escKey)
                        break;
                    end
                end
                if(isplaying(audio_player) == 0)
                    play(audio_player);
                end
            end
            stop(audio_player);
            Screen('DrawTexture',wPtr,textures.page38,[],pageRect);
            Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
            Screen('Flip', wPtr, 0);
            RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);
        case 3
            % Exp 3 Experience Actor2
            blockDim_str = "Experience";
            dim = 0;
            actor = "Actor2";
            actorID = 1;
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp(("Note to the executor:"+newline+...
                " The upcoming block is: " + blockDim_str + " " + actor +newline));
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp("Input required from the executor:"+newline+...
                ">> Press ESC key to continue.");
            Screen('FillRect',wPtr,background);
            Screen('DrawTexture',wPtr,textures.page37,[],pageRect);
            Screen('Flip', wPtr, 0);
            while 1
                [keyIsDown, secs, keyCode] = KbCheck;
                if keyIsDown
                    if keyCode(escKey)
                        break;
                    end
                end
                if(isplaying(audio_player) == 0)
                    play(audio_player);
                end
            end
            stop(audio_player);
            Screen('DrawTexture',wPtr,textures.page39,[],pageRect);
            Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
            Screen('Flip', wPtr, 0);
            RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);
        case 4
            blockDim_str = "Experience";
            actor = "Actor1";
            dim = 0;
            actorID = 0;
            Screen('FillRect',wPtr,background);
            Screen('DrawTexture',wPtr,textures.page37,[],pageRect);
            Screen('Flip', wPtr, 0);
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp(("Note to the executor:"+newline+...
                " The upcoming block is: " + blockDim_str + " " + actor +newline));
            disp(("-------------------------------------------------------------------------------------------"+ newline));
            disp("Input required from the executor:"+newline+...
                ">> Press ESC key to continue.");
            while 1
                [keyIsDown, secs, keyCode] = KbCheck;
                if keyIsDown
                    if keyCode(escKey)
                        break;
                    end
                end
                if(isplaying(audio_player) == 0)
                    play(audio_player);
                end
            end
            stop(audio_player);
            Screen('DrawTexture',wPtr,textures.page39,[],pageRect);
            Screen('DrawTexture', wPtr, textures.arrow_img,[],arrow);
            Screen('Flip', wPtr, 0);
            RecordMouse(wPtr,screenNum,[],[],arrow,importLocs,[],continue_mouseCallerID);
    end
    HideCursor;


    % Inside the following loop, the execution of a trial is implemented
    % for the number of trials in each block.
    for trialNum=1:size(actionOrder,1)
        [actionname, ~] = MatchIDtoClass(actionOrder(trialNum,blocks));
        write(machineServer,actionname); % Send the action name to the Actor PC
        Screen('FillRect',wPtr,white);
        Screen('DrawTexture',wPtr,textures.fixation,[],pageRect); % Draw fixation texture
        Screen('Flip',wPtr,0); % Display previously drawn texture
        trialStart = GetSecs;
        Screen('TextSize',wPtr, textsize);
        Screen('FillRect', wPtr, actionbackground); % draw the background color to action condition
        WaitSecs(fixationduration); % Wait for 2 seconds to flip the screen
        Screen('Flip',wPtr);
        disp(("-------------------------------------------------------------------------------------------"+ newline+newline));
        disp(("Note to the experimenter:"+newline+...
            "Action performed by the actor is..: "+newline));
        disp((">>" + upper(actionname)));
        % Turn on lights
        writeDigitalPin(arduinoLights,'D9',1); % Logic High Signal
        WaitSecs(stimulusDuration);
        % Turn off lights
        writeDigitalPin(arduinoLights,'D9',0);
        % Call GenerateResponsePage for response screen
        GenerateResponsePage(dim,wPtr,start_button, dimensionLow, dimensionHigh,dimensionTextures);
        [VBLt, ~] =Screen('Flip',wPtr);
        % Call mouse tracking
        respStart = GetSecs;
        [answer, points, timePoints] = RecordMouse(wPtr, screenNum, [], ...
            start_button,button, importLocs,dim,1);
        respEnd = GetSecs;
        HideCursor;
        % RECORD EXPERIMENTAL DATA
        data.ExperimentPart3(trialCounter).BlockDimension = blockDim_str;
        data.ExperimentPart3(trialCounter).BlockDimensionID = dim;
        data.ExperimentPart3(trialCounter).Actor = actor;
        data.ExperimentPart3(trialCounter).ActorID = actorID;
        data.ExperimentPart3(trialCounter).ExperimentPart = "Exp3";
        data.ExperimentPart3(trialCounter).BlockNo = experimentBlockOrder(blocks);
        data.ExperimentPart3(trialCounter).BlockType = actor + "_" + blockDim_str;
        data.ExperimentPart3(trialCounter).ActionID = actionOrder(trialNum,blocks);
        [data.ExperimentPart3(trialCounter).ActionName, data.ExperimentPart3(trialCounter).ActionClass] = MatchIDtoClass(actionOrder(trialNum,blocks));
        data.ExperimentPart3(trialCounter).responseTrajectory = points;
        data.ExperimentPart3(trialCounter).respTrajectoryTime = timePoints;
        data.ExperimentPart3(trialCounter).ResponseTime = respEnd - respStart;
        % Response check
        if strcmp(answer, "L")
            % top left corner --> high
            participantResp = "HighAgency";
            participantRespID = leftChoiceID;

        elseif strcmp(answer, "R")
            % top right corner --> low
            participantResp = "LowAgency";
            participantRespID = rightChoiceID;
        elseif strcmp(answer, "M")
            % missed trial
            participantResp = "Missed";
            participantRespID = 99;
        else
            % undefined area clicks
            participantResp = "Out";
            participantRespID = 99;
        end
        data.ExperimentPart3(trialCounter).Response = participantResp;
        data.ExperimentPart3(trialCounter).ResponseID = participantRespID;

        % Saving data to xlsx, definitions of excel columns
        BlockDimension =  data.ExperimentPart3(trialCounter).BlockDimension;
        BlockDimensionID = data.ExperimentPart3(trialCounter).BlockDimensionID;
        Actor = data.ExperimentPart3(trialCounter).Actor;
        ActorID = actorID;
        ExperimentPart =  data.ExperimentPart3(trialCounter).ExperimentPart;
        BlockType = data.ExperimentPart3(trialCounter).BlockType;
        ActionID = data.ExperimentPart3(trialCounter).ActionID;
        ActionName = data.ExperimentPart3(trialCounter).ActionName;
        ActionClass = data.ExperimentPart3(trialCounter).ActionClass;
        TrialDuration = respEnd - trialStart;
        ResponseTime =respEnd - respStart;
        Response = participantResp;
        ResponseID = participantRespID;
        cd(datapath);
        if trialCounter == 1
            % First input to the excel sheet
            experimentPart3Data = table(ExperimentPart,BlockDimension,BlockDimensionID,BlockType,Actor,ActorID,ActionID,ActionName,...
                ActionClass,TrialDuration, ResponseTime, Response,ResponseID);
            writetable(experimentPart3Data,experiment3_xlsx_file,'Sheet',1,"AutoFitWidth",false,'Range','A1');

        else
            experimentPart3Data_t  = table(ExperimentPart,BlockDimension,BlockDimensionID,BlockType,Actor,ActorID,ActionID,ActionName,...
                ActionClass,TrialDuration, ResponseTime, Response,ResponseID);

            writetable(experimentPart3Data_t,experiment3_xlsx_file,'WriteMode','Append',...
                'WriteVariableNames',false,'WriteRowNames',true);
        end
        cd(exppath);
        trialCounter = trialCounter + 1;
    end
    disp(("-------------------------------------------------------------------------------------------"+ newline+newline));
    disp(("Note to the executor:"+newline+...
        "Saving data... Please wait"+newline));
    cd(datapath)
    save(experiment3_mat_file,'-struct','data','ExperimentPart3','ParticipantInfo');
    cd(exppath)
    disp(("-------------------------------------------------------------------------------------------"+ newline+newline));
    disp(("Note to the executor:"+newline+...
        "Data is saved."+newline));

end


Screen('DrawTexture',wPtr,textures.page40,[],pageRect);
Screen('Flip', wPtr, 0);

% Experiment Part 3 is completed, notify the participant and exit

disp(("-------------------------------------------------------------------------------------------"+ newline+newline));
disp(("Note to the executor:"+newline+...
    "Experiment Part 3 is completed"+newline));
disp("Press ESC key to continue.");
while 1
    [keyIsDown, secs, keyCode] = KbCheck;
    if keyIsDown
        if keyCode(escKey)
            break ;
        end
    end
end
Screen('Close', wPtr); %close the screen
